@@ -1,4 +1,4 @@
-//@line 36 "/export/home/mozilla/uild/firefox1.5.0.2/mozilla/browser/components/preferences/cookies.js"
+//@line 39 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/preferences/cookies.js"
 
 const nsICookie = Components.interfaces.nsICookie;
 
@@ -22,11 +22,7 @@ var gCookiesWindow = {
     this._bundle = document.getElementById("bundlePreferences");
     this._tree = document.getElementById("cookiesList");
     
-    this._loadCookies();
-    this._tree.treeBoxObject.view = this._view;
-    this.sort("rawHost");
-    if (this._view.rowCount > 0) 
-      this._tree.view.selection.select(0);
+    this._populateList(true);
       
     document.getElementById("filter").focus();
   },
@@ -39,6 +35,30 @@ var gCookiesWindow = {
     os.removeObserver(this, "perm-changed");
   },
   
+  _populateList: function (aInitialLoad)
+  {
+    this._loadCookies();
+    this._tree.treeBoxObject.view = this._view;
+    if (aInitialLoad)
+      this.sort("rawHost");
+    if (this._view.rowCount > 0) 
+      this._tree.view.selection.select(0);
+
+    if (aInitialLoad) {
+      if ("arguments" in window && window.arguments[0] &&
+          window.arguments[0].filterString)
+      {
+        this.setFilter(window.arguments[0].filterString);
+      }
+    }
+    else {
+      if (document.getElementById("filter").value != "")
+        this.filter();
+    }
+
+    this._saveState();
+  },
+  
   _cookieEquals: function (aCookieA, aCookieB, aStrippedHost)
   {
     return aCookieA.rawHost == aStrippedHost &&
@@ -67,6 +87,13 @@ var gCookiesWindow = {
       this._tree.treeBoxObject.rowCountChanged(0, -oldRowCount);
       this._view.selection.clearSelection();
     }
+    else if (aData == "reload") {
+      // first, clear any existing entries
+      this.observe(aCookie, aTopic, "cleared");
+
+      // then, reload the list
+      this._populateList(false);
+    }
     
     // We don't yet handle aData == "deleted" - it's a less common case
     // and is rather complicated as selection tracking is difficult
@@ -151,7 +178,7 @@ var gCookiesWindow = {
     this._view._rowCount += rowCountImpact;
     this._tree.treeBoxObject.rowCountChanged(oldRowCount - 1, rowCountImpact);
 
-    document.getElementById("removeAllCookies").disabled = this._view._filtered || (this._view.rowCount == 0);
+    document.getElementById("removeAllCookies").disabled = this._view._filtered;
   },
   
   _view: {
@@ -227,6 +254,18 @@ var gCookiesWindow = {
     {
       var removeCount = aCount === undefined ? 1 : aCount;
       if (this._filtered) {
+        // remove the cookies from the unfiltered set so that they
+        // don't reappear when the filter is changed. See bug 410863.
+        for (var i = aIndex; i < aIndex + removeCount; ++i) {
+          var item = this._filterSet[i];
+          var parent = gCookiesWindow._hosts[item.rawHost];
+          for (var j = 0; j < parent.cookies.length; ++j) {
+            if (item == parent.cookies[j]) {
+              parent.cookies.splice(j, 1);
+              break;
+            }
+          }
+        }
         this._filterSet.splice(aIndex, removeCount);
         return;
       }
@@ -395,6 +434,10 @@ var gCookiesWindow = {
     { 
       return false; 
     },
+    isSelectable: function (aIndex, aColumn) 
+    { 
+      return false; 
+    },
     setCellValue: function (aIndex, aColumn, aValue) {},    
     setCellText: function (aIndex, aColumn, aValue) {},    
     performAction: function (aAction) {},  
@@ -478,9 +521,7 @@ var gCookiesWindow = {
   _updateCookieData: function (aItem)
   {
     var seln = this._view.selection;
-    var ids = ["nameLabel", "name", "valueLabel", "value", "isDomain", "host", 
-               "pathLabel", "path", "isSecureLabel", "isSecure", "expiresLabel", 
-               "expires"];
+    var ids = ["name", "value", "host", "path", "isSecure", "expires"];
     var properties;
     
     if (aItem && !aItem.container && seln.count > 0) {
@@ -537,13 +578,13 @@ var gCookiesWindow = {
     var stringKey = selectedCookieCount == 1 ? "removeCookie" : "removeCookies";
     document.getElementById("removeCookie").label = this._bundle.getString(stringKey);
     
-    document.getElementById("removeAllCookies").disabled = this._view._filtered || (this._view.rowCount == 0);
+    document.getElementById("removeAllCookies").disabled = this._view._filtered;
     document.getElementById("removeCookie").disabled = !(seln.count > 0);
   },
   
   deleteCookie: function () 
   { 
-//@line 628 "/export/home/mozilla/uild/firefox1.5.0.2/mozilla/browser/components/preferences/cookies.js"
+//@line 672 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/browser/components/preferences/cookies.js"
     var seln = this._view.selection;
     var tbo = this._tree.treeBoxObject;
     
@@ -686,8 +727,7 @@ var gCookiesWindow = {
     // Revert to single-select in the tree
     this._tree.setAttribute("seltype", "single");
     
-    // Clear the Filter and the Tree Display
-    document.getElementById("filter").value = "";
+    // Clear the Tree Display
     this._view._filtered = false;
     this._view._rowCount = 0;
     this._tree.treeBoxObject.rowCountChanged(0, -this._view._filterSet.length);
@@ -696,6 +736,17 @@ var gCookiesWindow = {
     // Just reload the list to make sure deletions are respected
     this._loadCookies();
     this._tree.treeBoxObject.view = this._view;
+    
+    // Restore sort order
+    var sortby = this._lastSortProperty;
+    if (sortby == "") {
+      this._lastSortAscending = false;
+      this.sort("rawHost");
+    }
+    else {
+      this._lastSortAscending = !this._lastSortAscending;
+      this.sort(sortby);
+    }
 
     // Restore open state
     for (var i = 0; i < this._openIndices.length; ++i)
@@ -711,7 +762,6 @@ var gCookiesWindow = {
     this._lastSelectedRanges = [];
 
     document.getElementById("cookiesIntro").value = this._bundle.getString("cookiesAll");
-    document.getElementById("clearFilter").disabled = true;
   },
   
   _cookieMatchesFilter: function (aCookie)
@@ -759,61 +809,49 @@ var gCookiesWindow = {
         this._openIndices.push(i);
     }
   },
-  
-  _filterTimeout: -1,
-  onFilterInput: function ()
-  {
-    if (this._filterTimeout != -1)
-      clearTimeout(this._filterTimeout);
-   
-    function filterCookies()
-    {
-      var filter = document.getElementById("filter").value;
-      if (filter == "") {
-        gCookiesWindow.clearFilter();
-        return;
-      }        
-      var view = gCookiesWindow._view;
-      view._filterSet = gCookiesWindow._filterCookies(filter);
-      if (!view._filtered) {
-        // Save Display Info for the Non-Filtered mode when we first
-        // enter Filtered mode. 
-        gCookiesWindow._saveState();
-        view._filtered = true;
-      }
-      // Move to multi-select in the tree
-      gCookiesWindow._tree.setAttribute("seltype", "multiple");
-      
-      // Clear the display
-      var oldCount = view._rowCount;
-      view._rowCount = 0;
-      gCookiesWindow._tree.treeBoxObject.rowCountChanged(0, -oldCount);
-      // Set up the filtered display
-      view._rowCount = view._filterSet.length;
-      gCookiesWindow._tree.treeBoxObject.rowCountChanged(0, view.rowCount);
-      
-      // if the view is not empty then select the first item
-      if (view.rowCount > 0)
-        view.selection.select(0);
 
-      document.getElementById("cookiesIntro").value = gCookiesWindow._bundle.getString("cookiesFiltered");
-      document.getElementById("clearFilter").disabled = false;
-    }
-    window.filterCookies = filterCookies;
-    this._filterTimeout = setTimeout("filterCookies();", 500);
-  },
-  
-  onFilterKeyPress: function (aEvent)
+  filter: function ()
   {
-    if (aEvent.keyCode == 27) // ESC key
-      this.clearFilter();
+    var filter = document.getElementById("filter").value;
+    if (filter == "") {
+      gCookiesWindow.clearFilter();
+      return;
+    }
+    var view = gCookiesWindow._view;
+    view._filterSet = gCookiesWindow._filterCookies(filter);
+    if (!view._filtered) {
+      // Save Display Info for the Non-Filtered mode when we first
+      // enter Filtered mode.
+      gCookiesWindow._saveState();
+      view._filtered = true;
+    }
+    // Move to multi-select in the tree
+    gCookiesWindow._tree.setAttribute("seltype", "multiple");
+
+    // Clear the display
+    var oldCount = view._rowCount;
+    view._rowCount = 0;
+    gCookiesWindow._tree.treeBoxObject.rowCountChanged(0, -oldCount);
+    // Set up the filtered display
+    view._rowCount = view._filterSet.length;
+    gCookiesWindow._tree.treeBoxObject.rowCountChanged(0, view.rowCount);
+
+    // if the view is not empty then select the first item
+    if (view.rowCount > 0)
+      view.selection.select(0);
+
+    document.getElementById("cookiesIntro").value = gCookiesWindow._bundle.getString("cookiesFiltered");
   },
-  
+
+  setFilter: function (aFilterString) {
+    document.getElementById("filter").value = aFilterString;
+    this.filter();
+  },
+
   focusFilterBox: function ()
-  { 
+  {
     var filter = document.getElementById("filter");
     filter.focus();
     filter.select();
   }
 };
-
