@@ -116,16 +116,19 @@ nsFilePicker.prototype = {
   set files(a) { throw "readonly property"; },
   get files()  { return this.mFilesEnumerator; },
 
-  /* readonly attribute nsIFileURL fileURL; */
+  /* readonly attribute nsIURI fileURL; */
   set fileURL(a) { throw "readonly property"; },
   get fileURL()  {
-    if (this.mFilesEnumerator) {
+    if (this.mFileURL)
+      return this.mFileURL;
+
+    if (!this.mFilesEnumerator)
+      return null;
+
       var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                     .getService(Components.interfaces.nsIIOService);
-      var url       = ioService.newFileURI(this.file);
-      return url;
-    }
-    return null;
+
+    return this.mFileURL = ioService.newFileURI(this.file);
   },
 
   /* attribute wstring defaultString; */
@@ -172,6 +175,7 @@ nsFilePicker.prototype = {
       this.appendFilter(bundle.GetStringFromName("xulTitle"),
                    bundle.GetStringFromName("xulFilter"));
     }
+    this.mAllowURLs = !!(filterMask & nsIFilePicker.filterAllowURLs);
     if (filterMask & nsIFilePicker.filterApps) {
       // We use "..apps" as a special filter for executable files
       this.appendFilter(bundle.GetStringFromName("appsTitle"),
@@ -207,6 +211,7 @@ nsFilePicker.prototype = {
     o.filters = new Object();
     o.filters.titles = this.mFilterTitles;
     o.filters.types = this.mFilters;
+    o.allowURLs = this.mAllowURLs;
     o.retvals = new Object();
 
     var parent;
@@ -247,6 +252,7 @@ nsFilePicker.prototype = {
 
       this.mFilterIndex = o.retvals.filterIndex;
       this.mFilesEnumerator = o.retvals.files;
+      this.mFileURL = o.retvals.fileURL;
       lastDirectory = o.retvals.directory;
       return o.retvals.buttonStatus;
     } catch(ex) { dump("unable to open file picker\n" + ex + "\n"); }
@@ -272,9 +278,9 @@ function (compMgr, fileSpec, location, type)
 
     compMgr.registerFactoryLocation(FILEPICKER_CID,
                                     "FilePicker JS Component",
-//@line 278 "/export/home/mozilla/uild/firefox1.5.0.2/mozilla/xpfe/components/filepicker/src/nsFilePicker.js.in"
+//@line 284 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/toolkit/components/filepicker/src/nsFilePicker.js.in"
                                     "",
-//@line 280 "/export/home/mozilla/uild/firefox1.5.0.2/mozilla/xpfe/components/filepicker/src/nsFilePicker.js.in"
+//@line 286 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/toolkit/components/filepicker/src/nsFilePicker.js.in"
                                     fileSpec,
                                     location,
                                     type);
