@@ -1,22 +1,24 @@
-//@line 44 "/export/home/mozilla/uild/firefox1.5.0.2/mozilla/toolkit/components/filepicker/content/filepicker.js"
+//@line 44 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/toolkit/components/filepicker/content/filepicker.js"
 
 const nsIFilePicker       = Components.interfaces.nsIFilePicker;
-const nsIDirectoryServiceProvider = Components.interfaces.nsIDirectoryServiceProvider;
-const nsIDirectoryServiceProvider_CONTRACTID = "@mozilla.org/file/directory_service;1";
+const nsIProperties       = Components.interfaces.nsIProperties;
+const NS_DIRECTORYSERVICE_CONTRACTID = "@mozilla.org/file/directory_service;1";
+const NS_IOSERVICE_CONTRACTID = "@mozilla.org/network/io-service;1";
 const nsITreeBoxObject = Components.interfaces.nsITreeBoxObject;
 const nsIFileView = Components.interfaces.nsIFileView;
-const nsFileView_CONTRACTID = "@mozilla.org/filepicker/fileview;1";
+const NS_FILEVIEW_CONTRACTID = "@mozilla.org/filepicker/fileview;1";
 const nsITreeView = Components.interfaces.nsITreeView;
 const nsILocalFile = Components.interfaces.nsILocalFile;
 const nsIFile = Components.interfaces.nsIFile;
-const nsLocalFile_CONTRACTID = "@mozilla.org/file/local;1";
-const nsIPromptService_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
+const NS_LOCAL_FILE_CONTRACTID = "@mozilla.org/file/local;1";
+const NS_PROMPTSERVICE_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
 
-var sfile = Components.classes[nsLocalFile_CONTRACTID].createInstance(nsILocalFile);
+var sfile = Components.classes[NS_LOCAL_FILE_CONTRACTID].createInstance(nsILocalFile);
 var retvals;
 var filePickerMode;
 var homeDir;
 var treeView;
+var allowURLs;
 
 var textInput;
 var okButton;
@@ -32,7 +34,7 @@ function filepickerLoad() {
 
   textInput = document.getElementById("textInput");
   okButton = document.documentElement.getButton("accept");
-  treeView = Components.classes[nsFileView_CONTRACTID].createInstance(nsIFileView);
+  treeView = Components.classes[NS_FILEVIEW_CONTRACTID].createInstance(nsIFileView);
 
   if (window.arguments) {
     var o = window.arguments[0];
@@ -49,6 +51,7 @@ function filepickerLoad() {
     const numFilters = filterTitles.length;
 
     document.title = title;
+    allowURLs = o.allowURLs;
 
     if (initialText) {
       textInput.value = initialText;
@@ -63,6 +66,7 @@ function filepickerLoad() {
   if (filePickerMode == nsIFilePicker.modeGetFolder) {
     var textInputLabel = document.getElementById("textInputLabel");
     textInputLabel.value = gFilePickerBundle.getString("dirTextInputLabel");
+    textInputLabel.accessKey = gFilePickerBundle.getString("dirTextInputAccesskey");
   }
   
   if ((filePickerMode == nsIFilePicker.modeOpen) ||
@@ -97,16 +101,13 @@ function filepickerLoad() {
     treeView.showOnlyDirectories = true;
   }
 
-  // set up the right icon
+  // The dialog defaults to an "open" icon, change it to "save" if applicable
   if (filePickerMode == nsIFilePicker.modeSave)
-    okButton.setAttribute("icon","save");
-  else
-    okButton.setAttribute("icon","open");
+    okButton.setAttribute("icon", "save");
 
   // start out with a filename sort
   handleColumnClick("FilenameColumn");
 
-  document.documentElement.setAttribute("ondialogcancel", "return onCancel();");
   try {
     setOKAction();
   } catch (exception) {
@@ -124,8 +125,7 @@ function filepickerLoad() {
 
   // Start out with the ok button disabled since nothing will be
   // selected and nothing will be in the text field.
-  okButton.disabled = true;
-  textInput.focus();
+  okButton.disabled = filePickerMode != nsIFilePicker.modeGetFolder;
 
   // This allows the window to show onscreen before we begin
   // loading the file list
@@ -135,17 +135,17 @@ function filepickerLoad() {
 
 function setInitialDirectory(directory)
 {
-  // get the home dir
-  var dirServiceProvider = Components.classes[nsIDirectoryServiceProvider_CONTRACTID]
-                                     .getService(nsIDirectoryServiceProvider);
-  var persistent = new Object();
-  homeDir = dirServiceProvider.getFile("Home", persistent);
+  // Start in the user's home directory
+  var dirService = Components.classes[NS_DIRECTORYSERVICE_CONTRACTID]
+                             .getService(nsIProperties);
+  homeDir = dirService.get("Home", Components.interfaces.nsIFile);
 
   if (directory) {
     sfile.initWithPath(directory);
+    if (!sfile.exists() || !sfile.isDirectory())
+      directory = false;
   }
-  if (!directory || !(sfile.exists() && sfile.isDirectory())) {
-    // Start in the user's home directory
+  if (!directory) {
     sfile.initWithPath(homeDir.path);
   }
 
@@ -174,7 +174,7 @@ function showErrorDialog(titleStrName, messageStrName, file)
   var errorMessage =
     gFilePickerBundle.getFormattedString(messageStrName, [file.path]);
   var promptService =
-    Components.classes[nsIPromptService_CONTRACTID].getService(Components.interfaces.nsIPromptService);
+    Components.classes[NS_PROMPTSERVICE_CONTRACTID].getService(Components.interfaces.nsIPromptService);
 
   promptService.alert(window, errorTitle, errorMessage);
 }
@@ -182,23 +182,9 @@ function showErrorDialog(titleStrName, messageStrName, file)
 function openOnOK()
 {
   var dir = treeView.selectedFiles.queryElementAt(0, nsIFile);
-  if (!dir.isReadable()) {
-    showErrorDialog("errorOpenFileDoesntExistTitle",
-                    "errorDirNotReadableMessage",
-                    dir);
-    return false;
-  }
-
   if (dir)
     gotoDirectory(dir);
 
-  retvals.fileList = new Array(dir);
-
-  retvals.buttonStatus = nsIFilePicker.returnCancel;
-  
-  var filterMenuList = document.getElementById("filterMenuList");
-  retvals.filterIndex = filterMenuList.selectedIndex;
-  
   return false;
 }
 
@@ -210,9 +196,28 @@ function selectOnOK()
   var isDir = false;
   var isFile = false;
 
-  var fileList = processPath(textInput.value);
+  retvals.filterIndex = document.getElementById("filterMenuList").selectedIndex;
+  retvals.fileURL = null;
 
-  if (!fileList) { // generic error message, should probably never happen
+  if (allowURLs) {
+    try {
+      var ios = Components.classes[NS_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
+      retvals.fileURL = ios.newURI(textInput.value, null, null);
+      var fileList = [];
+      if (retvals.fileURL instanceof Components.interfaces.nsIFileURL)
+        fileList.push(retvals.fileURL.file);
+      gFilesEnumerator.mFiles = fileList;
+      retvals.files = gFilesEnumerator;
+      retvals.buttonStatus = ret;
+
+      return true;
+    } catch (e) {
+    }
+  }
+
+  var fileList = processPath(textInput.value);
+  if (!fileList) {
+    // generic error message, should probably never happen
     showErrorDialog("errorPathProblemTitle",
                     "errorPathProblemMessage",
                     textInput.value);
@@ -290,8 +295,8 @@ function selectOnOK()
             gFilePickerBundle.getFormattedString("confirmFileReplacing",
                                                  [file.path]);
           
-          promptService = Components.classes[nsIPromptService_CONTRACTID].getService(Components.interfaces.nsIPromptService);
-          var rv = promptService.confirm(window, title, message);
+          promptService = Components.classes[NS_PROMPTSERVICE_CONTRACTID].getService(Components.interfaces.nsIPromptService);
+          var rv = promptService.confirm(window, confirmTitle, message);
           if (rv) {
             ret = nsIFilePicker.returnReplace;
             retvals.directory = file.parent.path;
@@ -332,7 +337,7 @@ function selectOnOK()
             errorMessage =
               gFilePickerBundle.getFormattedString("saveWithoutPermissionMessage_dir", [parent.path]);
           }
-          promptService = Components.classes[nsIPromptService_CONTRACTID].getService(Components.interfaces.nsIPromptService);
+          promptService = Components.classes[NS_PROMPTSERVICE_CONTRACTID].getService(Components.interfaces.nsIPromptService);
           promptService.alert(window, errorTitle, errorMessage);
           ret = nsIFilePicker.returnCancel;
         }
@@ -352,9 +357,6 @@ function selectOnOK()
 
   retvals.files = gFilesEnumerator;
   retvals.buttonStatus = ret;
-
-  var filterMenuList = document.getElementById("filterMenuList");
-  retvals.filterIndex = filterMenuList.selectedIndex;
   
   return (ret != nsIFilePicker.returnCancel);
 }
@@ -465,11 +467,10 @@ function onKeypress(e) {
 }
 
 function doEnabling() {
+  if (filePickerMode != nsIFilePicker.modeGetFolder)
   // Maybe add check if textInput.value would resolve to an existing
   // file or directory in .modeOpen. Too costly I think.
-  var enable = (textInput.value != "");
-
-  okButton.disabled = !enable;
+    okButton.disabled = (textInput.value == "")
 }
 
 function onTreeFocus(event) {
@@ -481,7 +482,7 @@ function setOKAction(file) {
   var buttonLabel;
   var buttonIcon = "open"; // used in all but one case
 
-  if (file && file.isDirectory() && filePickerMode != nsIFilePicker.modeGetFolder) {
+  if (file && file.isDirectory()) {
     document.documentElement.setAttribute("ondialogaccept", "return openOnOK();");
     buttonLabel = gFilePickerBundle.getString("openButtonLabel");
   }
@@ -543,7 +544,7 @@ function onFileSelected(/* nsIArray */ selectedFileList) {
   if (validFileSelected) {
     setOKAction(file);
     okButton.disabled = invalidSelection;
-  } else
+  } else if (filePickerMode != nsIFilePicker.modeGetFolder)
     okButton.disabled = (textInput.value == "");
 }
 
@@ -576,7 +577,7 @@ function onDirectoryChanged(target)
 {
   var path = target.getAttribute("label");
 
-  var file = Components.classes[nsLocalFile_CONTRACTID].createInstance(nsILocalFile);
+  var file = Components.classes[NS_LOCAL_FILE_CONTRACTID].createInstance(nsILocalFile);
   file.initWithPath(path);
 
   if (!sfile.equals(file)) {
@@ -631,7 +632,7 @@ function goHome() {
 function newDir() {
   var file;
   var promptService =
-    Components.classes[nsIPromptService_CONTRACTID].getService(Components.interfaces.nsIPromptService);
+    Components.classes[NS_PROMPTSERVICE_CONTRACTID].getService(Components.interfaces.nsIPromptService);
   var dialogTitle =
     gFilePickerBundle.getString("promptNewDirTitle");
   var dialogMsg =
@@ -706,11 +707,11 @@ function gotoDirectory(directory) {
 
   window.setCursor("auto");
 
-  treeView.QueryInterface(nsITreeView).selection.clearSelection();
   if (filePickerMode == nsIFilePicker.modeGetFolder) {
     textInput.value = "";
   }
   textInput.focus();
+  textInput.setAttribute("autocompletesearchparam", directory.path);
   sfile = directory;
 }
 
@@ -720,7 +721,7 @@ function toggleShowHidden(event) {
 
 // from the current directory and whatever was entered
 // in the entry field, try to make a new path. This
-// uses "/" as the directory seperator, "~" as a shortcut
+// uses "/" as the directory separator, "~" as a shortcut
 // for the home directory (but only when seen at the start
 // of a path), and ".." to denote the parent directory.
 // returns an array of the files listed,
@@ -785,7 +786,18 @@ function processPathEntry(path, fileArray)
   var filePath;
   var file;
 
-  if (path[0] == '~') 
+  try {
+    file = sfile.clone().QueryInterface(nsILocalFile);
+  } catch(e) {
+    dump("Couldn't clone\n"+e);
+    return false;
+  }
+
+  var tilde_file = file.clone();
+  tilde_file.append("~");
+  if (path[0] == '~' &&                        // Expand ~ to $HOME, except:
+      !(path == "~" && tilde_file.exists()) && // If ~ was entered and such a file exists, don't expand
+      (path.length == 1 || path[1] == "/"))    // We don't want to expand ~file to ${HOME}file
     filePath = homeDir.path + path.substring(1);
   else
     filePath = path;
@@ -793,13 +805,6 @@ function processPathEntry(path, fileArray)
   // Unescape quotes
   filePath = filePath.replace(/\\\"/g, "\"");
   
-  try{
-    file = sfile.clone().QueryInterface(nsILocalFile);
-  } catch(e) {
-    dump("Couldn't clone\n"+e);
-    return false;
-  }
-
   if (filePath[0] == '/')   /* an absolute path was entered */
     file.initWithPath(filePath);
   else if ((filePath.indexOf("/../") > 0) ||
