@@ -138,6 +138,8 @@ pref("media.enforce_same_site_origin", false);
 // Media cache size in kilobytes
 pref("media.cache_size", 51200);
 
+//@line 142 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/modules/libpref/src/init/all.js"
+pref("media.ogg.enabled", true);
 //@line 145 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/modules/libpref/src/init/all.js"
 pref("media.wave.enabled", true);
 //@line 147 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/modules/libpref/src/init/all.js"
