@@ -1,4 +1,4 @@
-//@line 36 "/export/home/mozilla/uild/firefox1.5.0.2/mozilla/toolkit/mozapps/preferences/removemp.js"
+//@line 38 "/export/home/ffslavex/mozilla-1.9.1-solaris-x86-nightly/build/toolkit/mozapps/preferences/removemp.js"
 
 var gRemovePasswordDialog = {
   _token    : null,
@@ -12,7 +12,7 @@ var gRemovePasswordDialog = {
                              .getService(Components.interfaces.nsIPromptService);
     this._bundle = document.getElementById("bundlePreferences");
 
-    this._okButton = document.documentElement.getButton("accept")
+    this._okButton = document.documentElement.getButton("accept");
     this._okButton.label = this._bundle.getString("pw_remove_button");
     
     this._password = document.getElementById("password");
